/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor;

import eu.agrosense.client.grid.Grid;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import nl.cloudfarming.client.geoviewer.BaseLayer;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.imp.ImportDataObject;
import nl.cloudfarming.client.imp.ImportProcessor;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.util.ExplorerManagerUtil;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectManager;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;

/**
 * A base DataObject for SensorData When constructed a background task is
 * started to process the content into a displayable image.
 *
 * @author Timon Veenstra
 */
@NbBundle.Messages({
    "sensor.palette.warning.save.palette=Could not save sensor palette",
    "# {0} - file name",
    "sensor.common.processorprovider.logerror.no_processor=No suitable processor found for file {0}"
})
public abstract class SensorDataObject extends ImportDataObject {

    private SensorPalette palette;
    private Grid grid;
    public static final String PROP_PALETTE_CHANGED = "paletteChanged";
    private static final String PALETTE_PROPERTY = "property";
    private static final AppLogger APP_LOG = AppLogFactory.getLogger(SensorDataObject.class);
    private BaseLayer layer;

    public SensorDataObject(FileObject fo, MultiFileLoader loader) throws DataObjectExistsException {
        super(fo, loader);
    }

    /**
     * Calls {@link MultiDataObject#createNodeDelegate()} And if not null adds
     * the result of {@link SensorDataObject#getSensorGrid()} and
     * {@link SensorDataObject#getLayer()} to the lookup
     *
     * @return
     */
    @Override
    protected Node createNodeDelegate() {
        //FIXME sensor grid construction should be done asynchronously
        ic.add(getLayer());
        ic.add(getSensorGrid());
        ic.add(getPalette());
        return super.createNodeDelegate();
    }

    /**
     * Get the sensor grid for a the {@link SensorProcessor} returned by
     * {@link SensorDataObject#getProcessor()}
     *
     * @return a grid if the file contains data
     */
    public final Grid getSensorGrid() {
        if (grid == null ) {
            ImportProcessor processor = getLookup().lookup(ImportProcessor.class);
            Project project = FileOwnerQuery.getOwner(getPrimaryFile());
            if (processor != null) {
                List<Grid> grids = new ArrayList<>();
                processor.<Grid>process(getPrimaryFile(), project, grids);
                if (grids.size() > 0) {
                    grid = grids.get(0);
                }
                ic.add(grid);
            } else {
                APP_LOG.log(Level.WARNING, Bundle.sensor_common_processorprovider_logerror_no_processor(getPrimaryFile().getNameExt()));
            }
        }
        return grid;
    }

    /**
     * Get the palette for the SensorDataObject
     *
     * @return
     */
    public SensorPalette getPalette() {
        if (palette == null) {
            palette = (SensorPalette) getPrimaryFile().getAttribute(PALETTE_PROPERTY);
            if (palette == null) {
                palette = SensorPalette.PALLET00_10;
            }
        }
        return palette;
    }

    private Layer getLayer() {
        if (layer == null) {
            layer = new SimpleLayer(getPrimaryFile().getName());
        }
        return layer;
    }

    /**
     * Set the palette for the SensorDataobject
     *
     * @param palette
     */
    public void setPalette(SensorPalette palette) {
        SensorPalette old = this.palette;
        this.palette = palette;
        getLayer().setPalette(palette);
        this.firePropertyChange(PROP_PALETTE_CHANGED, old, palette);

        try {
            getPrimaryFile().setAttribute(PALETTE_PROPERTY, palette);
        } catch (IOException ex) {
            APP_LOG.warning(Bundle.sensor_palette_warning_save_palette());
        }
    }
}
